! Author: P.Videler
! Version: 0.1
! 
! (c) 1997 European Space Agency 
!
! Description: Various image statistics
! 
! Comments:
! 
! Modifications: 
! 
! $Id: imagestat.f90,v 1.2 1999/04/08 13:41:38 pvideler Exp $
! 
module imageStat
 use saslib
 use obj_images
 use obj_imageStat
 use tableStat,        only : imageStat_table

 implicit none 

 !......................................................................
 private    ! change default to private
 public  :: &
            imageStatistics,   &
            imageStat_t
 !......................................................................

 interface imageStatistics
    module procedure imageStat_images, imageStat_table
 end interface

contains 

 ! ====================================================================
 function imageStat_images (xx) result(s)
  ! ====================================================================
  type(images_t),  intent(in)  :: xx
  type(imageStat_t)            :: s

  integer                      :: i,j, lb(2), ub(2), n

  lb = lbound(xx%image) 
  ub = ubound(xx%image)

  ! init stat results type
  call init(s)

  ! loop over all pixels.
  ! note: this routine avoids using F90 vector functions. For large images 
  !       it is probably more efficient to combine calculations in a single loop. 
  !       (avoids memory swapping)

  if (doBadpix(xx)) then           ! do badpix in separate loop (faster)

     do i=lb(1), ub(1)
        do j=lb(2), ub(2)
           ! if (doBadpix(xx)) then
           if (xx%badpix(i,j) /= 0) then
              s%nbadpix = s%nbadpix + 1
              cycle                              ! skip bad points
           endif
           ! endif
           s%total = s%total + xx%image(i,j)
           s%sumOfSquares = s%sumOfSquares + xx%image(i,j)**2
           s%sigma(1) = s%sigma(1) + i * xx%image(i,j)
           s%sigma(2) = s%sigma(2) + j * xx%image(i,j)
           if (xx%image(i,j) < s%minval ) then
              s%minval = xx%image(i,j)
              s%minpos = (/i,j/)
           endif
           if (xx%image(i,j) > s%maxval ) then
              s%maxval = xx%image(i,j)
              s%maxpos = (/i,j/)
           endif
        enddo
     enddo

  else

     do i=lb(1), ub(1)
        do j=lb(2), ub(2)
           s%total = s%total + xx%image(i,j)
           s%sumOfSquares = s%sumOfSquares + xx%image(i,j)**2
           s%sigma(1) = s%sigma(1) + i * xx%image(i,j)
           s%sigma(2) = s%sigma(2) + j * xx%image(i,j)
           if (xx%image(i,j) < s%minval ) then
              s%minval = xx%image(i,j)
              s%minpos = (/i,j/)
           endif
           if (xx%image(i,j) > s%maxval ) then
              s%maxval = xx%image(i,j)
              s%maxpos = (/i,j/)
           endif
        enddo
     enddo

  endif

  ! normalize center
  if (s%total == 0) then
     call warning ('imagelib/src/imagestat','Sum of image counts is zero. &
                    & Can not derive center position for this image. &
                    & Using default (-1,-1) for center')
  else 
    s%center = s%sigma / s%total
  endif

  s%ngoodpix = size(xx%image) - s%nbadpix

  ! variance
  n = s%ngoodpix
  if (n>0) then
    s%variance = (s%sumOfSquares - s%total**2 /n) /n
  else
   s%variance = 0
  endif 

  s%xrange = (/lb(1),ub(1)/)
  s%yrange = (/lb(2),ub(2)/)

 end function imageStat_images

end module imageStat

