! Author: P.Videler
! Version: 1.0
! 
! (c) 1997 European Space Agency 
!
! Description: Various image statistics
! 
! Comments:
! 
! Modifications: 
! 
! $Id: obj_imagestat.f90,v 1.3 1999/04/08 13:41:39 pvideler Exp $
! 
module obj_imageStat
 use saslib

 implicit none 

 !......................................................................
 private    ! change default to private
 public  :: &
      imageStat_t,        &
      init,               &
      report,             &
      retrieve,           &
      store
 !......................................................................

 type imageStat_t 
    real(kind=single)   :: sigma(2), center(2)
    real(kind=single)   :: total, sumOfSquares, variance
    real(kind=single)   :: minval, maxval
    integer             :: nbadpix, ngoodpix
    integer             :: minpos(2), maxpos(2), xrange(2), yrange(2)
 end type imageStat_t

 interface init
    module procedure initStat
 end interface

 interface report
    module procedure printStat
 end interface

 interface store
    module procedure saveStat
 end interface

 interface retrieve
    module procedure readStat
 end interface

contains 

 ! ====================================================================
 subroutine initStat(s)
  ! ====================================================================
  type(imageStat_t), intent(out)  :: s

  s%center   = -1         ! special value. 
  s%sigma    = 0
  s%total    = 0
  s%sumOfSquares = 0
  s%variance = 0
  s%ngoodpix = 0
  s%nbadpix  = 0
  s%minval   = +huge(1._single)
  s%maxval   = -huge(1._single)
  s%minpos   = -1
  s%maxpos   = -1
  s%xrange   = (/ +huge(1), -huge(1)/)
  s%yrange   = (/ +huge(1), -huge(1)/)

 end subroutine initStat

 ! ====================================================================
 subroutine printStat(s)
  ! ====================================================================
  type(imageStat_t), intent(in)  :: s
  character(len=1), parameter    :: lf=char(10) ! CR+LF

 write(errstr, * ) &
! write(errstr, '(  2(a,i7), 2(a,2(f10.4)), 4(a,2i7 ), 5(a,f10.4), a )' ) &

      ' Image statistics :'  // &
  lf// ' nGoodPix  = ',  s%ngoodpix, & !2(a,i)
  lf// ' nBadPix   = ',  s%nbadpix, & 

  lf// ' Center    = ',  s%center, &  !2(a 2f)  
  lf// ' Sigma     = ',  s%sigma , &

  lf// ' pos min   = ',  s%minpos, &  !4(a,2i)
  lf// ' pos max   = ',  s%maxpos, &
  lf// ' x range   = ',  s%xrange, &
  lf// ' y range   = ',  s%yrange, &

  lf// ' min Value = ',   s%minval, & !5(a,f)
  lf// ' max Value = ',   s%maxval, &
  lf// ' Sum       = ',   s%total, &
  lf// ' Std. Dev. = ',   sqrt(s%variance), &
  lf// ' Sum of Sq = ',   s%sumOfSquares, &

  lf// ' '
 
  call message(appLibMsg, Verbose)
  
 end subroutine printStat

 ! ====================================================================
 subroutine saveStat(s, set, tblname)
  ! ====================================================================
  ! store statistics info in existing dataSet
  use dal
  type(imageStat_t),   intent(in)    :: s
  type(DataSetT),      intent(inout) :: set
  character(len=*),    intent(in)    :: tblname
  type(TableT)                       :: tbl

  tbl = addTable(set, tblname, 0)   ! empty table

  call setAttribute(tbl, "xCenter"  , s%center(1) )
  call setAttribute(tbl, "yCenter"  , s%center(2) )
  call setAttribute(tbl, "xSigma"   , s%sigma(1)  )
  call setAttribute(tbl, "ySigma"   , s%sigma(2)  )
  call setAttribute(tbl, "Sum"      , s%total     )
  call setAttribute(tbl, "Variance" , s%variance  )
  call setAttribute(tbl, "SumOfSq"  , s%sumOfSquares)
  call setAttribute(tbl, "nGoodPix" , s%ngoodpix  )
  call setAttribute(tbl, "nBadPix"  , s%nbadpix   )
  call setAttribute(tbl, "minValue" , s%minval    )
  call setAttribute(tbl, "maxValue" , s%maxval    )
  call setAttribute(tbl, "xPosMin"  , s%minpos(1) )
  call setAttribute(tbl, "yPosMin"  , s%minpos(2) )
  call setAttribute(tbl, "xPosMax"  , s%maxpos(1) )
  call setAttribute(tbl, "yPosMax"  , s%maxpos(2) )

!!$  ###  Bug in DAL 1.1 requires max 8 char ### 
!!$  call setAttribute(tbl, "xRangeMin" , s%xrange(1) )
!!$  call setAttribute(tbl, "xRangeMax" , s%xrange(2) )
!!$  call setAttribute(tbl, "yRangeMin" , s%yrange(1) )
!!$  call setAttribute(tbl, "yRangeMax" , s%yrange(2) )

  call setAttribute(tbl, "xRangMin" , s%xrange(1) )
  call setAttribute(tbl, "xRangMax" , s%xrange(2) )
  call setAttribute(tbl, "yRangMin" , s%yrange(1) )
  call setAttribute(tbl, "yRangMax" , s%yrange(2) )


  call release(tbl)

 end subroutine saveStat

 ! ====================================================================
 function readStat(set, tblname) result(s)
  ! ====================================================================
  ! read statistics info from  existing dataSet
  use dal
  type(imageStat_t)                  :: s
  type(DataSetT),      intent(in)    :: set
  character(len=*),    intent(in)    :: tblname

  type(TableT)                       :: tbl

  call init(s)

  tbl = table(set, tblname)  

  s%center(1)   =  real32Attribute(tbl, "xCenter")
  s%center(2)   =  real32Attribute(tbl, "yCenter")
  s%sigma(1)    =  real32Attribute(tbl, "xSigma" )
  s%sigma(2)    =  real32Attribute(tbl, "ySigma" )
  s%total       =  real32Attribute(tbl, "Sum" )
  s%variance    =  real32Attribute(tbl, "Variance" )
  s%sumOfSquares = real32Attribute(tbl, "SumOfSq" )
  s%ngoodpix    =   int32Attribute(tbl, "nGoodPix")
  s%nbadpix     =   int32Attribute(tbl, "nBadPix" )
  s%minval      =  real32Attribute(tbl, "minValue")
  s%maxval      =  real32Attribute(tbl, "maxValue")
  s%minpos(1)   =   int32Attribute(tbl, "xPosMin")
  s%minpos(2)   =   int32Attribute(tbl, "yPosMin" )
  s%maxpos(1)   =   int32Attribute(tbl, "xPosMax")
  s%maxpos(2)   =   int32Attribute(tbl, "yPosMax")

!!$  ###  Bug in DAL 1.1 requires max 8 char ### 
!!$  s%xrange(1) =   int32Attribute(tbl, "xRangeMin")
!!$  s%xrange(2) =   int32Attribute(tbl, "xRangeMax")
!!$  s%yrange(1) =   int32Attribute(tbl, "yRangeMin")
!!$  s%yrange(2) =   int32Attribute(tbl, "yRangeMax")

  s%xrange(1) =   int32Attribute(tbl, "xRangMin")
  s%xrange(2) =   int32Attribute(tbl, "xRangMax")
  s%yrange(1) =   int32Attribute(tbl, "yRangMin")
  s%yrange(2) =   int32Attribute(tbl, "yRangMax")

  call release(tbl)

 end function readStat
 
end module obj_imageStat
