! Module: conversionExample.f90 
! Author: P.Videler
! Version: 0.9
! 
! (c) 1998 European Space Agency 
!
! Description: Example program to convert images between array form and table form.
!              part of the XMM Science Analysis System
! 
! Comments:
! 
! Modifications: 
! 
! $Id: imconversiontest_mod.f90,v 1.4 2007/06/11 13:48:50 rsaxton Exp $
! 
module imConversionTest_mod
 use saslib

 implicit none 

 private    ! change default to private
 public  :: imConversionTest

contains 

 ! ====================================================================
 subroutine imConversionTest
  ! ====================================================================
  use imagelib
  use datafilter

  type(imagesDal_t)                 :: aa(2)
  type(images_t), target      :: xx_t, xxout_t
  type(images_t), pointer      :: xx, xxout
  character(len=*),    parameter    :: fn='imConversionTest : '
  integer                           :: nselect, imsize(2)

  type(DataSetT)    :: eventsSet   
  type(TableT)      :: eventsTable 

  type(imageStat_t) :: s

  ! ----------
  ! Initialize 
  ! ----------

  call init(aa)      ! always needed for using imagesDal

  ! ---------------------------
  ! Get image pointers from DAL
  ! ---------------------------

  !$!  xx => getImages(aa, imageSetName=stringParameter("image"))

  xx_t = getImages(aa(1), imageSetName  = stringParameter("image")  &
       , imageName     = stringParameter("image")  &
       , badpixSetName = stringParameter("badpix") &
       , badpixName    = stringParameter("badpix") &
       , weightSetName = stringParameter("weight") &
       , weightName    = stringParameter("weight") &
       )

  xx => xx_t
  s = imageStatistics(xx)
  call report(s)

  ! -------------------
  !  convert to event list
  ! -------------------

  eventsSet = dataSet(stringParameter('events'), create)

  eventsTable = addImage2eventTable(xx, eventsSet, tblname='events')

  ! ----------------------
  !  filter event table 
  ! ----------------------

  if (booleanParameter('filter')) then
     call initFilter(stringParameter("filterexpression"))
     nselect = doSelect(eventsTable, &
           flagColumnName=stringParameter("filterflagcolumn") )
    write(*,*) fn// 'number of selected events=', nselect
  endif

  ! ----------------------
  !  test table2image conversion 
  ! ----------------------

  ! scan table to find dimensions needed for image 

  s = imageStatistics(eventsTable)
  call report(s)
  imsize(1) = 1 + s%xrange(2) - s%xrange(1)
  imsize(2) = 1 + s%yrange(2) - s%yrange(1)

  ! setup new image

  xxout_t = createImages(aa(2),                      &
       imsize = imsize,            &
       imageSetName = stringParameter("set"), &
       imageName = 'pvtest')

  xxout => xxout_t

  xxout%lowleft = (/s%xrange(1) , s%yrange(1)/)

  !  call initTable2Image(aa(2), eventsTable, xxout, binsize, min, max)

  call addTable2Image(eventsTable, xxout)

  ! statistics on newly created image
  s = imageStatistics(xxout)
  call report(s)


  ! ------------
  !   Compare
  ! ------------

  if (ANY(xx%image /= xxout%image)) call error('images do not match')

  if (dobadpix(xx)) then
     if (ANY(xx%badpix /= xxout%badpix)) call error('badpiximages do not match')
  endif

  if (doweight(xx)) then
     if (ANY(xx%weight /= xxout%weight)) call error('weight images do not match')
  endif

  ! ------------
  !   Clean up
  ! ------------

  call free(aa)

  call release(eventsTable)
  call release(eventsSet)

 end subroutine imConversionTest

end module 













